package com.jeeplus.test.reimbursementsys.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.jeeplus.aop.demo.annotation.DemoMode;
import com.jeeplus.aop.logging.annotation.ApiLog;
import com.jeeplus.sys.utils.StringUtils;
import com.jeeplus.test.reimbursementsys.domain.dto.*;
import com.jeeplus.test.reimbursementsys.service.WuHanReimbursementBusinessService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * @author 徐滕
 * @version 1.0.0
 * @ClassName ReimbursementUserController.java
 * @Description TODO 报销人员年份金额信息
 * @createTime 2022年05月16日 10:35:21
 */
@RestController
@RequestMapping("/wuHanReimbursementBusiness")
public class WuHanReimbursementBusinessController {

    @Autowired
    private WuHanReimbursementBusinessService service;

    /**
     * 根据id报销人员信息
     * @param id
     * @return
     */
    @ApiLog("根据id报销人员信息")
    @GetMapping("queryById")
    public ResponseEntity queryById(@RequestParam String id) {
        ReimbursementBusinessDTO reimbursementDTO = service.queryById (id);
        return ResponseEntity.ok (reimbursementDTO);
    }

    /**
     * 根据id报销人员信息
     * @param id
     * @return
     */
    @ApiLog("根据id报销人员信息")
    @PreAuthorize("hasAnyAuthority('wuHanReimbursementBusiness:add')")
    @GetMapping("getBusinessById")
    public ResponseEntity getBusinessById(@RequestParam String id) {
        WuHanReimbursementDTO reimbursementDTO = service.getWuHanBusinessById (id);
        return ResponseEntity.ok (reimbursementDTO);
    }

    /**
     * 根据人员id查询报销年份信息
     * @param reimbursementUser
     * @param page
     * @return
     */
    @ApiLog("查询角色列表")
    @GetMapping("getYearList")
    public ResponseEntity<IPage<ReimbursementUserDTO>> getYearList(ReimbursementUserDTO reimbursementUser, Page<ReimbursementUserDTO> page) {
        IPage<ReimbursementUserDTO> list = service.getYearList(page,reimbursementUser);
        return ResponseEntity.ok (list);
    }


    /**
     * 查询报销信息
     * @return
     */
    @ApiLog("查询报销人员信息列表")
    @GetMapping("findList")
    public ResponseEntity findList(ReimbursementUserDTO reimbursementUser, Page<ReimbursementUserDTO> page) {
        IPage<ReimbursementUserDTO> list = service.findList(page,reimbursementUser);
        return ResponseEntity.ok(list);
    }

    /**
     * 保存业务报销编码
     * @return
     */
    @ApiLog("保存报销人员信息")
    @PostMapping("save")
    public ResponseEntity save(@RequestBody ReimbursementBusinessDTO reimbursementUser) {
        Map<String, Object> result = service.save(reimbursementUser);
        Boolean success = (Boolean) result.get("success");
        if(success){
            return ResponseEntity.ok(result.get("message"));
        }
        return ResponseEntity.badRequest ().body (result.get("message"));
    }

    /**
     * 批量删除
     * @param ids
     * @return
     */
    @DemoMode
    @ApiLog("批量删除")
    @PreAuthorize("hasAuthority('wuHanReimbursementBusiness:del')")
    @DeleteMapping("delete")
    public ResponseEntity delete(String ids) {
        String idArray[] =ids.split(",");
        service.delete ( Lists.newArrayList (idArray) );
        return ResponseEntity.ok ("删除成功！");
    }




    /**
     * 查询报销信息
     * @return
     */
    @ApiLog("查询业务报销信息列表")
    @PreAuthorize("hasAuthority('wuHanReimbursementBusiness:view')")
    @GetMapping("findListByBusinessCodeId")
    public ResponseEntity findListByBusinessCodeId(String id) {
        if(StringUtils.isBlank(id)){
            return ResponseEntity.badRequest ().body ("请先选择业务信息");
        }
        ReimbursementBusinessDTO reimbursementBusiness = new ReimbursementBusinessDTO();
        reimbursementBusiness.setBusinessCodeId(id);
        ReimbursementBusinessReturnDTO info = service.findListByBusinessCodeId(reimbursementBusiness);
        return ResponseEntity.ok(info);
    }
}
